<?php

namespace Drupal\inline_formatter_field\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'html_inject_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "inline_formatter_field_formatter",
 *   label = @Translation("Inline Formatter Field Formatter"),
 *   field_types = {
 *     "inline_formatter_field"
 *   }
 * )
 */
class InlineFormatterFieldFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'formatted_field' => '<h1>Hello World!</h1>',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $form['#attached']['library'][] = 'inline_formatter_field/ace_editor';

    $form['formatted_field'] = [
      '#title' => $this->t('HTML or Twig Format'),
      '#type' => 'textarea',
      '#default_value' => $this->getSetting('formatted_field'),
      '#description' => $this->t("Enter any html or Twig here along with token patterns."),
      '#attributes' => [
        'class' => ['AceEditorTextarea'],
      ],
    ];

    if (\Drupal::moduleHandler()->moduleExists('token')) {
      $form['token_help'] = [
        '#theme' => 'token_tree_link',
        '#token_types' => 'all',
        '#global_types' => FALSE,
        '#dialog' => TRUE,
      ];
    }

    // var_dump($this->getBaseId());
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $summary_format = $this->getSetting('formatted_field');
    if (strlen($summary_format) > 350) {
      $summary_format = substr($summary_format, 0, 350) . " ...";
    }

    $summary[] = $summary_format;
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];
    $settings = $this->getSettings();
    $formatted_field = $settings['formatted_field'];

    foreach ($items as $delta => $item) {
      // Get entity.
      $entity = $item->getEntity();

      // If the token module is enabled then do token replacement.
      $moduleHandler = \Drupal::service('module_handler');
      if ($moduleHandler->moduleExists('token')) {
        $token_data = [
          $entity->getEntityTypeId() => $entity,
        ];
        $formatted_field = \Drupal::token()->replace($formatted_field, $token_data, ['clear' => FALSE]);
      }

      // Uses the inline template feature from Drupal to be able to use Twig.
      $element[$delta] = [
        '#type' => 'inline_template',
        '#template' => $formatted_field,
        '#context' => [
          'node' => $entity,
        ],
      ];
    }

    return $element;
  }

}
